package net.sergeych.bintools

import kotlin.reflect.typeOf

/**
 * data input stream-like abstraction. We need it because
 * kotlinx serialization is synchronous and there us nothing
 * like multiplatform version of DataInput
 *
 */
interface DataSource {

    class EndOfData() : Exception("no more data available")

    fun readByte(): Byte


    fun readUByte() = readByte().toUByte()

    @Suppress("unused")
    fun readBytes(size: Int): ByteArray =
        ByteArray(size).also { a ->
            for (i in 0 until size)
                a[i] = readByte()
        }

    fun readU32(): UInt = bytesToUInt(readBytes(4))
    fun readI32(): Int = bytesToInt(readBytes(4))
    fun readI64(): Long = bytesToLong(readBytes(8))

    fun readDouble() = Double.fromBits(readI64())
    fun readFloat() = Float.fromBits(readI32())

}

fun ByteArray.toDataSource(): DataSource =
    object : DataSource {
        var position = 0
            private set

        override fun readByte(): Byte =
            if( position < size ) this@toDataSource[position++]
        else throw DataSource.EndOfData()
    }

inline fun <reified T : Any> DataSource.readNumber(): T = when(typeOf<T>()) {
    typeOf<Double>() -> readDouble() as T
    typeOf<Float>() -> readFloat() as T
    else -> Smartint.decode(this)
}
