package net.sergeych.bipack

import kotlinx.serialization.SerialInfo

/**
 * Serializable classes annotated as Framed will have leading checked mark as CRC32
 * of its name (uses `@SerialName` internally). On deserializing, if frame will not
 * match the expected name, the [InvalidFrameException] will be thrown.
 */
@SerialInfo
@Target(AnnotationTarget.CLASS)
annotation class Framed

class InvalidFrameException : Exception("invalid CRC32 serialization frame value")
